# -*- coding:utf-8 -*-
import os, subprocess, re
import tempfile
import shutil
import platform

from utils import print_err
import subprocess
from log import log_debug, log_error, LogManager

grub_key="GRUB_CMDLINE_LINUX_SECURITY"
grub_path="/etc/default/grub"
sw_grub_path="/boot/boot/grub/grub.cfg"

def update_boot_params(params):
	if platform.processor() == "sw_64":
		'''
		更新申威Grub配置文件

		参数:
		params: 安全启动参数，格式为字符串

		返回值:
		操作成功返回(True, None)，失败返回（False, 失败信息)
		'''
		file_data = ""
		start = False
		try:
			with open(sw_grub_path, "r", encoding="utf-8") as f:
				for line in f:
					if platform.release() in line:
						start = True
					if start == True:
						if "linux.vmlinux" in line:
							matchObj = re.search( r'selinux=(.)', line, re.M|re.I)
							if matchObj:
								line = line.replace(matchObj.group(),"")
							matchObj = re.search( r'enforcing=(.)', line, re.M|re.I)
							if matchObj:
								line = line.replace(matchObj.group(),"")
							matchObj = re.search( r'security=(.* )', line, re.M|re.I)
							if matchObj:
								line = line.replace(matchObj.group(),"")
							matchObj = re.search( r'kysec_status=(.)', line, re.M|re.I)
							if matchObj:
								line = line.replace(matchObj.group(),"")
							matchObj = re.search( r'kysec_3adm=(.)', line, re.M|re.I)
							if matchObj:
								line = line.replace(matchObj.group(),"")
							matchObj = re.search( r'security=', line, re.M|re.I)
							if matchObj:
								line = line.replace(matchObj.group(),"")
							line = line.rstrip() + " " + params + "\n"
							start = False
					file_data += line
		except Exception as e:
			log_error("Can't open grub configuration file for read: %s"%str(e))
			return (False, "无法打开grub默认配置")
		try:
			with open(sw_grub_path,"w",encoding="utf-8") as f:
				f.write(file_data)
		except Exception as e:
			log_error("Can't open grub configuration file for write: %s"%str(e))
			return (False, "无法写入配置文件")
	else:
		'''
		更新Grub启动配置, 该函数会先修改/etc/default/grub配置
		然后调用update-grub命令更新启动配置。

		参数:
		params: 安全启动参数，格式为字符串

		返回值:
		操作成功返回(True, None)，失败返回（False, 失败信息)
		'''
		try:
			f = open(grub_path, "r")		
		except Exception as e:
			log_error("Can't open grub configuration file for read: %s"%str(e))
			return (False, "无法打开grub默认配置")

		try:
			fd, temp_path = tempfile.mkstemp(prefix="grub.")
			temp = open(temp_path, "w")
		except Exception as e:
			f.close()
			log_error("Can't create temp file for write: %s"%str(e))
			return (False, "无法写入临时文件")

		exists=False
		while True:
			line = f.readline()
			if not len(line):
				break
			if not line.startswith("%s="%grub_key):
				temp.write(line)
				continue

			temp.write("%s=\"%s\"\n"%(grub_key, params))
			exists = True

		# grub security args not found, just append
		if not exists:
			temp.write("%s=\"%s\"\n"%(grub_key, params))

		f.close() 
		temp.close()

		try:
			shutil.move(temp_path, grub_path)
			log_debug("Updating boot configuration file...")
			retcode, grub_cfg = subprocess.getstatusoutput("find /boot -name grub.cfg")
			if retcode == 0:
				for cfg in grub_cfg.split('\n'):
					args = "grub2-mkconfig -o %s"%cfg
					subprocess.check_call(args, stderr=LogManager.getfd(), shell=True)
			else:
				args = "grub2-mkconfig -o /boot/grub2/grub.cfg"
				subprocess.check_call(args, stderr=LogManager.getfd(), shell=True)
		except subprocess.CalledProcessError as e:
			log_error("failed to call update-grub: %s"%str(e))
			return (False, "运行grub2-mkconfig命令时出错")
		except Exception as e:
			log_error("can't save grub configuration: %s"%str(e))
			return (False, "保存grub配置文件时出错")

	return (True, None)
